// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PASS_H
#define PASS_H

#include <vector>

#include "nrtypes_nr.h"

class Pass {

 public:

  Pass(int gs, int passno, int s);

  void addPoint(double obt, double ert);

  int  getStart() { return Start; }

  void setEnd(int End) { this->End = End; }
  int  getEnd() { return End; }

  void setPassNumber(int n) { passNumber = n; }
  int  getPassNumber() { return passNumber; }

  int  getGSID() { return GSID; }
 
  double getPredictedERT(double obt);

  double getRMS() { return rms; }

  void setDataOffset(int offs) { this->dataStartOffset = offs; }

  // Jump from previous pass to this one
  double jump;

  // Correction to be made
  double symmetricCorrection;

  // Does this pass overlap the target revolution
  bool inRev;

  double getGradient() { return gradient; }
  double getConstant() { return constant; }
 
 private:

  // Points
  vector<double> ERT;
  vector<double> OBT;
  vector<double> sigma;

  // Fit has been calculated
  bool fitCalculated;

  // Ground Station
  int GSID;

  // Pass in revolution
  int passNumber;

  // Start Frame
  int Start;

  // End Frame
  int End;

  // RMS
  double rms;

  long double gradient;
  long double constant;

  int dataStartOffset;

  void calculateFit();
  void calculateLSFit(int startOffset);
  bool calculateMRQFit(int startOffset);


};


#endif
