// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// CRx14 jcv 17-Apr-09 Infrastructure for tmprocessing (dumper/fixer) 
// SPRx14 jcv 21-Apr-09 enhanced dump functionality: mandatory start-end pars

#include <iostream>
#include <sstream>
#include <fstream>

#include <vector>

#include "DUExtractor.h"
#include "NCTRSProcessor.h"
#include "PacketProcessor.h"
#include "STSPProcessor.h"

#include "tmproc.h"
#include "Revolution.h"

#include "errstr.h"
#include "ExceptionErrorHandler.h"

///////////////////////
//
//
//
//
void tmproc () {

  bool bS2K = booleanParameter("useS2K");
  string sCacheDir = stringParameter("cachedir");
  bool bStrictRun=booleanParameter("strict");

  if(  bStrictRun  ) {
    errstr << "Strict=yes Running in normal mode" << Sas::warning("Strict_yes");
    errstr << "Strict=yes Running in normal mode" << Sas::message(Msg::App, Msg::Verbose);
  } else {
    errstr << "Strict=no WARNING Running in non-abort mode" << Sas::warning("Strict_no");
    errstr << "Strict=no WARNING Running in non-abort mode: ONLY GOOD DUs will be dumped. " << Sas::message(Msg::App, Msg::Verbose);
  }

  errstr << "S2K boolean is (1=TRUE 0=FALSE): " << bS2K << Sas::message(Msg::App, Msg::Verbose);
  errstr << "TM cache:" << sCacheDir << Sas::message(Msg::App, Msg::Verbose);

  string sOutFile = stringParameter("outfile");
  string sRawFile = stringParameter("rawfile");

  // NEw SPRx14
  STime start = stringParameter("StartTime");
  STime end  = stringParameter("EndTime");
  // END SPRx14

  string sFullName;
  FILE * pRawFile;
  FILE * pOutFile;

  int maxDUsize;

  // NEw SPRx14 Start-End no longer read from filename, but mandatory
  //   string sDate;
  //   string sStart;
  //   sDate=sRawFile.substr(5,23);
  //   sDate=sDate.replace(4,1,"-");
  //   sDate=sDate.replace(7,1,"-");
  //   sDate=sDate.replace(13,1,":");
  //   sDate=sDate.replace(16,1,":");
  //   sStart=sDate;

  //  start=STime(sStart);

  // We will use approx. boundaries for being sure the whole file is read
  //if (start.hour() >= 12) { 
  // start=STime(start.year(),start.month(),start.day(),11,59,0);
  // end=STime(start.year(),start.month(),1+start.day(),0,1,0);
  //}
  //else {
  //  end=STime(start.year(),start.month(),start.day(),12,1,0);
  //  start=STime(start.year(),start.month(),start.day()-1,23,59,0);
  //}
  // END SPRx14

  errstr << "Data time range is " << start << " to " << end << Sas::message(Msg::App, Msg::Sparse);

  // For Frames checking this is the only thing needed
  DUExtractor myExtractor( start, end );

  // For packet checking, this is also needed
    STSPProcessor proc = STSPProcessor(0,0);
    proc.Start = start;
    proc.End   = end;
    NCTRSProcessor * myNCTRSProcessor = new NCTRSProcessor(&proc, NCTRSProcessor::VC0);
    PacketProcessor * myPacketProcessor = new PacketProcessor(&proc);
    myExtractor.registerProcessor(myNCTRSProcessor);
    myNCTRSProcessor->registerProcessor(myPacketProcessor);

  // We compare against the default value found in tmcheck.par 
  errstr << "To read " << sRawFile << Sas::message(Msg::App, Msg::Verbose);
  sFullName = sCacheDir;
  sFullName.append("/");
  sFullName.append(sRawFile);

  errstr << "Full filename to read " << sFullName << Sas::message(Msg::App, Msg::Verbose);

  if ((pRawFile = fopen(sFullName.c_str(),"r")) == NULL) {
      errstr << "Failed to open data file " << sFullName 
             << Sas::fatal("failOpenDatafile");
    }

  errstr << "To write " << sOutFile << Sas::message(Msg::App, Msg::Verbose);
  sFullName = sCacheDir;
  sFullName.append("/");
  sFullName.append(sOutFile);

  errstr << "Full filename to write " << sFullName << Sas::message(Msg::App, Msg::Verbose);
  if ((pOutFile = fopen(sFullName.c_str(),"wb")) == NULL) {
      errstr << "Failed to open outputfile " << sFullName 
             << Sas::fatal("failOpenOutfile");
    }

  // Read DUs by opening just the requested file
  myExtractor.dumpDU(pRawFile,pOutFile, &maxDUsize,bStrictRun);

  fclose(pRawFile);

  errstr << "Biggest DU found " << dec << maxDUsize << Sas::message(Msg::App, Msg::Sparse);


// End of program

}
